// vim:syntax=c
constant __author = "Arne Goedeke <el@laramies.com>";
constant __version = "0.1";

constant js_html = "<script type=\"text/javascript\"\n   src=\"http://api.recaptcha.net/challenge?k=%s\">\n</script>\n <noscript>\n   <iframe src=\"http://api.recaptcha.net/noscript?k=%s\"\n       height=\"300\" width=\"500\" frameborder=\"0\"></iframe><br>\n   <textarea name=\"recaptcha_challenge_field\" rows=\"3\" cols=\"40\">\n   </textarea>\n   <input type=\"hidden\" name=\"recaptcha_response_field\" \n       value=\"manual_challenge\">\n</noscript>";

/* 
 * Pike reCAPTCHA implementation.
 * Copyright (C) 2007 Arne Goedeke
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 */

import Protocols.HTTP;

Standards.URI verify_url = Standards.URI("http://api-verify.recaptcha.net/verify");

//! Create a reCAPTCHA client instance for a specific public/private keypair. 
//! The client object may then be used to verify captchas and generate the html
//! to be included into your website.
//! 
//! Visit @[http://www.recaptcha.org] for information about reCAPTCHA.
class Client(string private_key, string public_key) {

    //! @[Protocols.HTTP.Session] object used internally.
    Session session = Session();

    //! Verifys a captcha. @expr{callback@} is called with either just 1 if the 
    //! captcha was entered correctly or 0 as the first and an error code as 
    //! the second argument. Check @[http://recaptcha.net/apidocs/captcha/] 
    //! for an explanation for what those error codes mean.
    //! @param remote_ip
    //! 	The ip adress of the user that entered the captcha.
    //! @param challenge
    //! 	The value of the @expr{recaptcha_challenge_field@} field.
    //! @param response
    //! 	The value of the @expr{recaptcha_response_field@} field.
    //! @param callback
    //!		To be called on success/failure. 	
    void verify(string remote_ip, string challenge, string response, function callback) {
	Session.Request request;

	if (!(stringp(challenge) && stringp(response) && sizeof(challenge) && sizeof(response))) {
	    call_out(callback, 0, 0, "incorrect-captcha-sol");
	    return;
	}

	void fail() {
	    callback(0, "recaptcha-not-reachable");
	};

	void data_ok() {
	    string data = request->data();
	    string line1, line2;
	    
	    [line1, line2] = data/"\n";

	    if (line1 == "true") {
		callback(1);
	    } else if (line1 == "false") {
		callback(0, line2);
	    } else {
		callback(0, "unknown");
	    }
	};

	request = session->async_post_url(verify_url, 
					 ([
					  "privatekey" : private_key,
					  "remoteip" : remote_ip,
					  "challenge" : challenge,
					  "response" : response,
					  ]),
					 0,
					 data_ok,
					 fail);
    }

    //! Generate html that shows the captcha to be included into the website. 
    //! The html is generated according to the API specifications at 
    //! @[http://recaptcha.net/apidocs/captcha/client.html].
    //! @param error_code
    //! 	Optional argument to include error messages into the captcha field.
    //! 	Use the error returned by the verify callback of the users last try.
    string get_js(string|void error_code) {
	string t = public_key;

	if (stringp(error_code) && sizeof(error_code)) {
	    t += "&error=" + error_code;
	}

	return sprintf(js_html, t, t);
    }
}
